module Main where

import System.Environment  
import TRS
import Parser
import LinearGWPO
import MaxPlusGWPO

-- wrapper
data GWPO = LinearGWPO (LinearGWPO.Order)
          | MaxPlusGWPO (MaxPlusGWPO.Order)

instance Show GWPO where
  show (LinearGWPO o) = show o
  show (MaxPlusGWPO o) = show o

linearGWPO :: String -> TRS -> IO (Maybe GWPO)
linearGWPO tool trs = do
  m <- LinearGWPO.linearGWPO tool trs
  case m of
    Just o -> return (Just (LinearGWPO o))
    Nothing -> return Nothing

linearWPO :: String -> TRS -> IO (Maybe GWPO)
linearWPO tool trs = do
  m <- LinearGWPO.linearWPO tool trs
  case m of
    Just o -> return (Just (LinearGWPO o))
    Nothing -> return Nothing

kbo :: String -> TRS -> IO (Maybe GWPO)
kbo tool trs = do
  m <- LinearGWPO.kbo tool trs
  case m of
    Just o -> return (Just (LinearGWPO o))
    Nothing -> return Nothing

maxPlusGWPO :: String -> TRS -> IO (Maybe GWPO)
maxPlusGWPO tool trs = do
  m <- MaxPlusGWPO.maxPlusGWPO tool trs
  case m of
    Just o -> return (Just (MaxPlusGWPO o))
    Nothing -> return Nothing

maxPlusGWPOZ :: String -> TRS -> IO (Maybe GWPO)
maxPlusGWPOZ tool trs = do
  m <- MaxPlusGWPO.maxPlusGWPOZ tool trs
  case m of
    Just o -> return (Just (MaxPlusGWPO o))
    Nothing -> return Nothing

maxPlusWPO :: String -> TRS -> IO (Maybe GWPO)
maxPlusWPO tool trs = do
  m <- MaxPlusGWPO.maxPlusWPO tool trs
  case m of
    Just o -> return (Just (MaxPlusGWPO o))
    Nothing -> return Nothing

lpo :: String -> TRS -> IO (Maybe GWPO)
lpo tool trs = do
  m <- MaxPlusGWPO.lpo tool trs
  case m of
    Just o -> return (Just (MaxPlusGWPO o))
    Nothing -> return Nothing

type Method = TRS -> IO (Maybe GWPO)

process :: Method -> TRS -> IO ()
process method trs = do
  result <- method trs
  case result of
    Nothing    -> putStrLn "MAYBE"
    Just order -> do
      putStrLn "YES"
      putStrLn ""
      putStrLn "TRS:"
      putStrLn ""
      putStr   (showTRS trs)
      putStrLn ""
      putStr   (show order)

run :: Method -> String -> IO ()
run method filename = do
      errorOrTRS <- readSpecFile filename
      case errorOrTRS of
        Left e    -> putStrLn (show e)
        Right trs -> process method trs

main :: IO ()
main = do
  args <- getArgs
  case args of
    ["-lpo", filename] -> run (Main.lpo tool) filename
    ["-kbo",   filename] -> run (Main.kbo tool) filename
    ["-wpol",  filename] -> run (Main.linearWPO tool) filename
    ["-wpomp", filename] -> run (Main.maxPlusWPO tool) filename
    ["-gwpol", filename] -> run (Main.linearGWPO tool) filename
    ["-gwpomp", filename] -> run (Main.maxPlusGWPOZ tool) filename
    _ -> do 
      putStrLn "gwpo <option> <filename>"
      putStrLn ""
      putStrLn "Options:"
      putStrLn "  -lpo     LPO"
      putStrLn "  -kbo     KBO"
      putStrLn "  -wpol    WPO with linear interpretations with 0,1-coefficients"
      putStrLn "  -wpomp   WPO with max/plus interpretations with 0,1-coefficients"
      putStrLn "  -gwpol   GWPO with linear interpretations with 0,1-coefficients"
      putStrLn "  -gwpomp  GWPO with max/plus interpretations with 0,1-coefficients"
      putStrLn ""
  where tool = "z3"
