module SymbolicPolynomial where

import Data.List
import SMT

-- (e0, [(x1, e1), ..., (xn, en)]) represents
-- e0 + e1 x1 + ... + en xn
-- assumes no duplicate variables such as [..., (x, ei), ..., (x, ej), ...]
type SymbolicPolynomial = (Exp, [(String, Exp)])

var :: String -> SymbolicPolynomial
var x = (Val 0, [(x, Val 1)])

const :: Exp -> SymbolicPolynomial
const e = (e, [])

addConstant :: Exp -> SymbolicPolynomial -> SymbolicPolynomial
addConstant c (e0, xes) = (plus [c, e0], xes)

mult :: Formula -> SymbolicPolynomial -> SymbolicPolynomial
mult f (e0, xes) = (times01 f e0, [ (x, times01 f e) | (x, e) <- xes ])

constantPart :: SymbolicPolynomial -> Exp
constantPart (e, _) = e

coef :: String -> SymbolicPolynomial -> Exp
coef x (_, xes)
  | Just e <- lookup x xes = e
  | otherwise = Val 0

-- a0 + a1 x1 + ... + an xn >= b0 + b1 y1 + ... + bm ym
encodeGeq :: SymbolicPolynomial -> SymbolicPolynomial -> Formula
encodeGeq pa@(_, xas) pb@(_, ybs) = 
  conj (SMT.geq (constantPart pa) (constantPart pb) : [ SMT.geq (coef z pa) (coef z pb) | z <- zs ])
  where
    zs = nub ([ x | (x, _) <- xas] ++ [ y | (y, _) <- ybs])

encodeGt :: SymbolicPolynomial -> SymbolicPolynomial -> Formula
encodeGt pa@(_, xas) pb@(_, ybs) = 
  conj (SMT.gt (constantPart pa) (constantPart pb) : [ SMT.geq (coef z pa) (coef z pb) | z <- zs ])
  where
    zs = nub ([ x | (x, _) <- xas] ++ [ y | (y, _) <- ybs])
